// sameproc.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "sameproc.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "sameprocDoc.h"
#include "sameprocView.h"
#include "AppWnd.h"
#include "AppThread.h"

#include <afxmt.h>
 
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSameprocApp

BEGIN_MESSAGE_MAP(CSameprocApp, CWinApp)
	//{{AFX_MSG_MAP(CSameprocApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_NEWINSTANCE, OnFileNewinstance)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSameprocApp construction

CSameprocApp::CSameprocApp()
	: CWinApp(_T("SameProc Sample"))
{

}

/////////////////////////////////////////////////////////////////////////////
// The one and only CSameprocApp object

CSameprocApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CSameprocApp initialization

BOOL CSameprocApp::InitInstance()
{
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	SetRegistryKey(_T("Codeguru"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)
	
	CWnd* pWnd = FindOrCreateAppWnd();

	if (!pWnd) //Creation failed
		return FALSE; //Terminate safely
	
	//m_pMainWnd is NULL if either we are not the first instance
	//or if window creation failed somehow
	if (m_pMainWnd == NULL)
	{
		SendCopyDataMsg(pWnd);	
		return FALSE;
	}
	
	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.
	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_SAMEPRTYPE,
		RUNTIME_CLASS(CSameprocDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CSameprocView));
	AddDocTemplate(pDocTemplate);

	
	//Finally send a message to create a thread
	SendCopyDataMsg(pWnd);
	
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CSameprocApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CSameprocApp message handlers



BOOL CSameprocApp::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	
	//The current thread should have the first preference

	CWinThread* pThread = AfxGetThread();

	if (pThread != this) //Not the main thread
	{
		if (pThread->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
			return TRUE;
	}

	return CWinApp::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CSameprocApp::OnFileNewinstance() 
{
	SendCopyDataMsg(m_pMainWnd);	
}

CWnd* CSameprocApp::FindOrCreateAppWnd()
{
	static TCHAR szMutexName[] = _T("SameProc");

	//Allow only one process to go through this function
	CMutex mtx(FALSE, szMutexName);
	CSingleLock lck(&mtx, TRUE);

	CAppWnd* pWnd = CAppWnd::FindAppWnd();

	if (!pWnd)
	{
		//Create the hidden window specific to the App 
		pWnd = new CAppWnd();

		if (!pWnd->CreateAppWnd())
		{
			delete pWnd;
			pWnd = NULL;
		}
		
		m_pMainWnd = pWnd;
	}
	
	return pWnd;
}

void CSameprocApp::SendCopyDataMsg(CWnd* pWnd)
{
	//Send the command line info to the other application
	COPYDATASTRUCT cds;

	cds.dwData = m_nCmdShow ;

	cds.cbData		 = _tcslen(m_lpCmdLine ) + 1;
	cds.lpData       = reinterpret_cast<LPVOID>(m_lpCmdLine);
	
	pWnd->SendMessage(WM_COPYDATA, 0, (LPARAM)(&cds));

}

